package org.codehaus.xfire.java;

import java.util.StringTokenizer;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.XFire;
import org.codehaus.xfire.fault.SOAP11FaultHandler;
import org.codehaus.xfire.handler.SoapHandler;
import org.codehaus.xfire.java.mapping.TypeMappingRegistry;
import org.codehaus.xfire.java.wsdl.JavaWSDLBuilder;

/**
 * Helper methods for services.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Nov 6, 2004
 */
public class ServiceHelper
{
    public static void createService( XFire xfire,
                                      TypeMappingRegistry registry,
                                      Class serviceClass )
    	throws Exception
    {
        DefaultJavaService service = new DefaultJavaService();
        
        String name = serviceClass.getName();
        int last = name.lastIndexOf(".");
        if ( last != -1 )
        {
            name = name.substring(last+1);
        }
        
        service.setName(name);
        service.setDefaultNamespace(makeNamespaceFromClassName(serviceClass.getName(), "http"));
        try
        {
            service.setServiceClass(serviceClass.getName());
        } catch (ClassNotFoundException e) {}
        
        service.setSoapVersion(SOAPConstants.SOAP11_ENVELOPE_NS);
        service.setStyle(SOAPConstants.STYLE_WRAPPED);
        service.setUse(SOAPConstants.USE_LITERAL);
        service.setFaultHandler(new SOAP11FaultHandler());
        service.setWSDLBuilder(new JavaWSDLBuilder( xfire.getTransportManager() ));
        service.setAutoTyped(true);
        
        SoapHandler handler = new SoapHandler(new JavaServiceHandler());
        service.setServiceHandler( handler );
        
        service.setTypeMappingRegistry(registry);
        
        service.initialize();
        
        xfire.getServiceRegistry().register(service);
    }
    
    public static String makeNamespaceFromClassName(String className, String protocol)
	{
        String packageName = className.substring(0, className.lastIndexOf("."));
        
		if ((packageName == null) || packageName.equals(""))
		{
			return protocol + "://" + "DefaultNamespace";
		}

		StringTokenizer st = new StringTokenizer(packageName, ".");
		String[] words = new String[st.countTokens()];

		for (int i = 0; i < words.length; ++i)
		{
			words[i] = st.nextToken();
		}

		StringBuffer sb = new StringBuffer(80);

		for (int i = words.length - 1; i >= 0; --i)
		{
			String word = words[i];

			// seperate with dot
			if (i != words.length - 1)
			{
				sb.append('.');
			}

			sb.append(word);
		}

		return protocol + "://" + sb.toString();
	}
}
