package org.codehaus.xfire.java.message;

import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.java.JavaService;
import org.codehaus.xfire.java.Operation;
import org.codehaus.xfire.java.type.Type;
import org.codehaus.xfire.util.DepthXMLStreamReader;

/**
 * Reads Document/Literal style messages.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class WrappedBridge
    extends AbstractMessageBridge
{
    public WrappedBridge( JavaService service, 
                          MessageContext context,
                          XMLStreamReader request,
                          XMLStreamWriter response )
    {
        super( service, context, request, response );
        
        setNamespace( request.getNamespaceURI() );
        String methodName = request.getLocalName();

        setOperation( getService().getOperation( methodName, getNamespace() ) );
	}
    
    public List read() throws XFireFault
    {
        List parameters = new ArrayList();
        LiteralReader reader = new LiteralReader(new DepthXMLStreamReader(getRequestReader()));
        Operation op = getOperation();

        if ( op == null )
        {
            throw new XFireFault("Invalid operation.", XFireFault.SENDER);
        }

        while( reader.hasMoreChildReaders() )
        {
            MessageReader cReader = reader.getNextChildReader();

            Class clazz = op.getParameterClass( cReader.getLocalName() );
            
            if ( clazz == null )
            {
                throw new XFireFault("Invalid parameter: " + cReader.getName(), XFireFault.SENDER);
            }
            
            Type type = getTypeMapping().getType( clazz );

            parameters.add( type.readObject( cReader ) );
        }
        
        return parameters;
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageBridge#write(java.lang.Object)
     */
    public void write(Object value)
    {
        String name = getOperation().getName() + "Response";
        LiteralWriter writer = new LiteralWriter( getResponseWriter(), 
                name, 
                getService().getDefaultNamespace() );
        
        Type type = getTypeMapping().getType( value.getClass() );
        String ns = null;
        
        if ( type.isComplex() )
        {
            ns = type.getSchemaType().getNamespace().getURI();
        }
        else
        {
            ns = getNamespace();
        }
        
        MessageWriter retWriter = writer.getChildWriter( getOperation().getOutParameterName(), 
                                                         ns );
        System.out.println("namespace: " + ns );
        type.writeObject( value, retWriter );
        
        retWriter.close();
        writer.close();
    }
}
