package org.codehaus.xfire.java.document;

import javax.xml.namespace.QName;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.fault.SOAP11FaultHandler;
import org.codehaus.xfire.handler.SoapHandler;
import org.codehaus.xfire.java.AbstractJavaTest;
import org.codehaus.xfire.java.DefaultJavaService;
import org.codehaus.xfire.java.JavaServiceHandler;
import org.codehaus.xfire.java.test.BeanService;
import org.codehaus.xfire.java.test.SimpleBean;
import org.codehaus.xfire.java.type.BeanType;
import org.codehaus.xfire.java.wsdl.JavaWSDLBuilder;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class BeanServiceTest
    extends AbstractJavaTest
{
    public void setUp() throws Exception
    {
        super.setUp();
        
        DefaultJavaService service = new DefaultJavaService();
        service.setName("Bean");
        service.setDefaultNamespace("urn:Bean");
        service.setServiceClass(BeanService.class.getName());
        service.setSoapVersion(SOAPConstants.SOAP11_ENVELOPE_NS);
        service.setStyle(SOAPConstants.STYLE_DOCUMENT);
        service.setUse(SOAPConstants.USE_LITERAL);
        service.setFaultHandler(new SOAP11FaultHandler());
        service.setWSDLBuilder(new JavaWSDLBuilder( getXFire().getTransportManager() ));
        
        SoapHandler handler = new SoapHandler(new JavaServiceHandler());
        
        service.setServiceHandler( handler );
        
        service.setTypeMappingRegistry(getRegistry());
        
        service.initialize();
        
        service.getTypeMapping().register(
                SimpleBean.class, 
                new QName("urn:Bean", "SimpleBean"), 
                BeanType.class );
        
        getServiceRegistry().register(service);
    }
    
    public void testBeanService() throws Exception
    {
        Document response = 
            invokeService( "Bean", "/org/codehaus/xfire/java/document/bean11.xml" );

        addNamespace( "sb", "urn:Bean" );
        assertValid( "//sb:out", response );
        assertValid( "//sb:out/sb:howdy[text()=\"howdy\"]", response );
        assertValid( "//sb:out/sb:bleh[text()=\"bleh\"]", response );
    }
    
    public void testBeanServiceWSDL() throws Exception
    {
        // Test WSDL generation
        Document doc = getWSDLDocument( "Bean" );

        addNamespace( "wsdl", WSDL.WSDL11_NS );
        addNamespace( "wsdlsoap", WSDL.WSDL11_SOAP_NS );
        addNamespace( "xsd", SOAPConstants.XSD );

        assertValid( "/wsdl:definitions/wsdl:types", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:complexType", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:complexType[@name=\"SimpleBean\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@name=\"bleh\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@name=\"howdy\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@type=\"xsd:string\"]", doc );

        assertValid( "/wsdl:definitions/wsdl:service/wsdl:port/wsdlsoap:address[@location='http://localhost/services/Bean']", doc );
    }
}
