package org.codehaus.xfire.java.wrapped;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.fault.SOAP11FaultHandler;
import org.codehaus.xfire.handler.SoapHandler;
import org.codehaus.xfire.java.AbstractJavaTest;
import org.codehaus.xfire.java.DefaultJavaService;
import org.codehaus.xfire.java.JavaServiceHandler;
import org.codehaus.xfire.java.ServiceHelper;
import org.codehaus.xfire.java.test.ArrayService;
import org.codehaus.xfire.java.test.BeanService;
import org.codehaus.xfire.java.wsdl.JavaWSDLBuilder;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;


/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class WrappedAutoTypeTest
    extends AbstractJavaTest
{
    public void setUp() throws Exception
    {
        super.setUp();
        ServiceHelper.createService(getXFire(), getRegistry(), BeanService.class);
       
        // register array service too
        DefaultJavaService service = new DefaultJavaService();
        service.setName("Array");
        service.setDefaultNamespace("urn:Array");
        service.setServiceClass(ArrayService.class.getName());
        service.setSoapVersion(SOAPConstants.SOAP11_ENVELOPE_NS);
        service.setStyle(SOAPConstants.STYLE_WRAPPED);
        service.setUse(SOAPConstants.USE_LITERAL);
        service.setFaultHandler(new SOAP11FaultHandler());
        service.setWSDLBuilder(new JavaWSDLBuilder( getXFire().getTransportManager() ));
        service.setAutoTyped(true);
        
        SoapHandler handler = new SoapHandler(new JavaServiceHandler());
        service.setServiceHandler( handler );
        
        service.setTypeMappingRegistry(getRegistry());
        
        service.initialize();
        
        getServiceRegistry().register(service);
    }
    
    public void testBeanService() throws Exception
    {
        Document response = 
            invokeService("BeanService", 
                          "/org/codehaus/xfire/java/wrapped/bean11.xml");

        addNamespace( "sb", "http://test.java.xfire.codehaus.org");
        assertValid( "//sb:getSimpleBeanResponse", response );
        assertValid( "//sb:getSimpleBeanResponse/sb:out", response );
        assertValid( "//sb:getSimpleBeanResponse/sb:out/sb:howdy[text()=\"howdy\"]", response );
        assertValid( "//sb:getSimpleBeanResponse/sb:out/sb:bleh[text()=\"bleh\"]", response );
    }

    public void testBeanServiceWSDL() throws Exception
    {
        Document doc = getWSDLDocument("BeanService");
        printNode(doc);
        addNamespace( "wsdl", WSDL.WSDL11_NS );
        addNamespace( "wsdlsoap", WSDL.WSDL11_SOAP_NS );
        addNamespace( "xsd", SOAPConstants.XSD );

        assertValid( "/wsdl:definitions/wsdl:types", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']", doc );
        assertValid( "//xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']/xsd:element[@name='getSubmitBean']", doc );
        assertValid( "//xsd:element[@name='getSubmitBean']/xsd:complexType/xsd:sequence/xsd:element[@name='in1'][@type='xsd:string']", doc );
        assertValid( "//xsd:element[@name='getSubmitBean']/xsd:complexType/xsd:sequence/xsd:element[@name='in0'][@type='tns:SimpleBean']", doc );
        
        assertValid( "/wsdl:definitions/wsdl:types" +
                "/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']" +
                "/xsd:complexType", doc );
        assertValid( "/wsdl:definitions/wsdl:types" +
                "/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']" +
                "/xsd:complexType[@name=\"SimpleBean\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types" +
                "/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']" +
                "/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@name=\"bleh\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types" +
                "/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']" +
                "/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@name=\"howdy\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types" +
                "/xsd:schema[@targetNamespace='http://test.java.xfire.codehaus.org']" +
                "/xsd:complexType[@name=\"SimpleBean\"]/xsd:sequence/xsd:element[@type=\"xsd:string\"]", doc );

        assertValid( "/wsdl:definitions/wsdl:service/wsdl:port/wsdlsoap:address[@location='http://localhost/services/BeanService']", doc );
    }
    
    public void testArrayService() throws Exception
    {
        Document response = invokeService("Array", 
                "/org/codehaus/xfire/java/wrapped/SubmitStringArray11.xml");

        addNamespace( "a", "urn:Array");
        addNamespace( "sb", "http://test.java.xfire.codehaus.org");
        assertValid( "//a:SubmitStringArrayResponse", response );
        assertValid( "//a:SubmitStringArrayResponse/a:out[text()='true']", response );
    }

    public void testArrayServiceWSDL() throws Exception
    {
        Document doc = getWSDLDocument("Array");
        
        addNamespace( "wsdl", WSDL.WSDL11_NS );
        addNamespace( "wsdlsoap", WSDL.WSDL11_SOAP_NS );
        addNamespace( "xsd", SOAPConstants.XSD );

        assertValid( "/wsdl:definitions/wsdl:types", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema[@targetNamespace='urn:Array']", doc );
        assertValid( "//xsd:schema[@targetNamespace='urn:Array']/xsd:element[@name='SubmitBeanArray']", doc );
        assertValid( "//xsd:element[@name='SubmitStringArray']/xsd:complexType/xsd:sequence/xsd:element[@name='in0'][@type='tns:ArrayOfString']", doc );
        assertValid( "//xsd:element[@name='SubmitBeanArray']/xsd:complexType/xsd:sequence/xsd:element[@name='in0'][@type='ns1:ArrayOfSimpleBean']", doc );
    }
}
